#include <UnitTest++.h>
#include "formule.hpp"
#include "parser.hpp"
#include "dimacs.hpp"
#include <iostream>
#include <sstream>
#include <fstream>

// Un test simple qui réussit
TEST(test1) {
  CHECK(2 == 1+1);
}

// Un test simple qui échoue
TEST(test2) {
  CHECK(2 == 1+2);
}

// Exemple de test des fonctions de lecture et d'écriture au format dimacs
TEST(lit_ecrit_dimacs) {
    cnf_t cnf;
    cnf.push_back({1,3,4,7});
    cnf.push_back({2,3});
    ostringstream out;
    for(auto it = cnf.cbegin(); it != cnf.cend(); ++it) {
        ecrit_clause_dimacs(out,*it);
    }
    istringstream in(out.str());
    dimacs d;
    lit_dimacs(in,d);
    auto it1 = cnf.cbegin();
    auto it2 = d.cnf.cbegin();
    while (it1 != cnf.cend() && it2 != d.cnf.cend()) {
        CHECK(*it1 == *it2);
        ++it1;
        ++it2;
    }
    CHECK(d.nbVars == 4);
}

TEST(lit_probleme1) {
  ifstream input("test-data/pb-simple1.txt");
  probleme pb = lit_probleme(input);
  CHECK(pb.nb_enseignants == 1);
  CHECK(pb.nb_salles == 1);
  CHECK(pb.nb_creneaux == 2);
  CHECK(pb.nb_cours == 2);
  CHECK(pb.enseigne[0] == set<int>({1,0}));
  CHECK(pb.salles[0] == set<int>({0}));
  CHECK(pb.salles[1] == set<int>({0}));
  CHECK(pb.indisponibilites[0] == set<int>({0}));
}

TEST(lit_probleme2) {
  ifstream input("test-data/pb-simple2.txt");
  probleme pb = lit_probleme(input);
  CHECK(pb.nb_enseignants == 2);
  CHECK(pb.nb_salles == 1);
  CHECK(pb.nb_creneaux == 2);
  CHECK(pb.nb_cours == 2);
  CHECK(pb.enseigne[0] == set<int>({1,0}));
  CHECK(pb.salles[0] == set<int>({0}));
  CHECK(pb.salles[1] == set<int>({0}));
  CHECK(pb.indisponibilites[0] == set<int>({0,1}));
}

TEST(lit_probleme4) {
  ifstream input("test-data/pb-simple4.txt");
  probleme pb = lit_probleme(input);
  CHECK(pb.nb_enseignants == 2);
  CHECK(pb.nb_salles == 2);
  CHECK(pb.nb_creneaux == 2);
  CHECK(pb.nb_cours == 3);
  CHECK(pb.enseigne[0] == set<int>({1,0,2}));
  CHECK(pb.enseigne[1] == set<int>({1,0}));
  CHECK(pb.salles[0] == set<int>({0}));
  CHECK(pb.salles[1] == set<int>({0}));
  CHECK(pb.salles[2] == set<int>({0,1}));
  CHECK(pb.indisponibilites[0] == set<int>({0,1}));
}

int main()
{
  return UnitTest::RunAllTests(); // un peu magique: lance tous les tests
}
